@@ -4,6 +4,9 @@
 #ifndef WIN32
 #define WIN32
 #endif
+#ifndef _CRT_SECURE_NO_DEPRECATE
+#define _CRT_SECURE_NO_DEPRECATE 1
+#endif /* _CRT_SECURE_NO_DEPRECATE */
 #include <winsock2.h>
 #include <mswsock.h>
 #include <ws2tcpip.h>
@@ -22,10 +25,8 @@
 #define snprintf _snprintf
 #if _MSC_VER < 1500
 #define vsnprintf _vsnprintf
-#else
-#define ssize_t SSIZE_T
-#define uint32_t UINT32
 #endif
+#define strdup _strdup
 #define strncasecmp _strnicmp
 #define strcasecmp _stricmp
 #else
@@ -33,12 +34,8 @@
 #define strcasecmp stricmp
 #endif /* _MSC_VER */
 
-/* Compile in zlib support */
-#define LIBSSH2_HAVE_ZLIB 1
-
 /* Enable newer diffie-hellman-group-exchange-sha1 syntax */
 #define LIBSSH2_DH_GEX_NEW 1
 
 #endif /* LIBSSH2_CONFIG_H */
 
-
